import json
from openai import OpenAI
from .tool import get_tool_schemas, get_tool_function
from typing import Any, Dict, List, Tuple, Generator

def get_tools(tags: list[str] = None) -> Tuple[List[Dict[str, Any]], Dict[str, Dict[str, Any]]]:
    """OpenAI tool schema wrapper with optional tag filtering."""
    functions = get_tool_schemas()

    tools = []
    available_functions = {}

    for function in functions:
        # If tags are provided, only include functions that match at least one tag
        function_tags = function.get("tags", [])

        if tags:
            if not any(tag in function_tags for tag in tags):
                continue  # Skip this function if none of its tags match

        tools.append({
            "type": "function",
            "function": {
                "name": function["name"],
                "description": function["description"],
                "parameters": function["parameters"],
                "return_type": function.get("return_type", "string"),  # Optional fallback
            },
        })
        
        func_name = function["name"]
        available_functions[func_name] = get_tool_function(func_name)

    return tools, available_functions

class OpenAITooling:
    def __init__(self, api_key: str = None, model: str = None, tool_choice: str = "auto"):
        self.api_key = api_key
        self.model = model
        self.tool_choice = tool_choice

    def call_tools(
            self,
            messages: list[dict[str, str]],
            api_key: str = None,
            model: str = None,
            tool_choice: str = "auto",
            tags: list[str] = None
        ) -> list[dict[str, str]]:
        """Non-streaming: returns full list of results."""
        
        result = self._internal_call_tools(
            messages=messages,
            api_key=api_key,
            model=model,
            tool_choice=tool_choice,
            tags=tags
        )
        if isinstance(result, Generator):
            return list(result)
        return result

    def stream_tools(
            self,
            messages: list[dict[str, str]],
            api_key: str = None,
            model: str = None,
            tool_choice: str = "auto",
            tags: list[str] = None
        ) -> Generator[str, None, None]:
        """Streaming version: yields partial results."""
        
        def generate():
            result = self._internal_call_tools(
                messages=messages,
                api_key=api_key,
                model=model,
                tool_choice=tool_choice,
                tags=tags
            )

            if isinstance(result, Generator):
                for partial in result:
                    if isinstance(partial, dict):
                        yield f"data: {json.dumps(partial)}\n\n"
                    elif isinstance(partial, str):
                        if partial.startswith("{") and partial.endswith("}"):
                            yield f"data: {partial}\n\n"
                        else:
                            yield f"data: {json.dumps({'response': partial})}\n\n"
                    else:
                        yield f"data: {json.dumps(partial)}\n\n"
            else:
                yield f"data: {json.dumps({'response': result})}\n\n"
        
        return generate()

    def _internal_call_tools(
            self,
            messages: list[dict[str, str]],
            api_key: str = None,
            model: str = None,
            tool_choice: str = "auto",
            tags: list[str] = None
        ):
        """Internal: original call logic (could still be generator)."""

        if not api_key:
            api_key = self.api_key
        if not model:
            model = self.model

        client = OpenAI(api_key=api_key)

        tools, available_functions = get_tools(tags=tags)

        completion = client.chat.completions.create(
            model=model,
            messages=messages,
            tools=tools,
            tool_choice=tool_choice,
        )
        response = completion.choices[0].message
        tool_calls = response.tool_calls
        
        if tool_calls:
            for tool_call in tool_calls:
                name = tool_call.function.name
                args = json.loads(tool_call.function.arguments)
                function_to_call = available_functions[name]

                if "messages" in args:
                    del args["messages"]

                result = function_to_call(**args, messages=messages)

                if isinstance(result, Generator):
                    for item in result:
                        yield item
                else:
                    messages.append({
                        "role": "function",
                        "tool_call_id": tool_call.id,
                        "name": name,
                        "content": result
                    })
                    yield messages
        else:
            raise ValueError("No tool calls found in the response.")

        